package controller.gui;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.ArrayList;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.table.DefaultTableModel;

import controller.Controller;
import driver.DriverConfig;
import driver.ExternalFileWorkloadPhase;
import driver.SyntheticWorkloadPhase;
import driver.WorkloadPhase;

/**
 * GUI for configuration of Drivers
 * 
 * @author Marcelo R.N. Mendes
 *
 */
@SuppressWarnings("serial")
public class DriverDetail extends ComponentDetail {
		
	//private ArrayList<WorkloadPhase> phases;
	private DriverConfig oldCfg;
	
    private JTextField addressField;
    private JLabel addressLabel;
    private JTextField aliasField;
    private JLabel aliasLabel;
    private JButton cancelBtn;
    private JLabel phasesLabel;
    private JScrollPane jScrollPane1;
    private JButton okBtn;
    private JTable phasesTable;
    private JPanel workloadPanel;
	private JCheckBox outOfOrderCheckBox;
	JPopupMenu phasesPop = new JPopupMenu();
	
    private JComboBox samplingComboBox;
    private JLabel samplingLabel;
    private JTextField serverAddressField;
    private JLabel serverAddressLabel;
    private JTextField serverPortField;
    private JLabel serverPortLabel;
    private JCheckBox validateCheckBox;
    private JPanel validationPanel;
    private JTextField validatorAddressField;
    private JLabel validatorAddressLabel;
    private JTextField validatorPortField;
    private JLabel validatorPortLabel;
		
    private ArrayList<WorkloadPhase> phases= new ArrayList<WorkloadPhase>();
 
    public DriverDetail(DriverConfig dr) {
    	initComponents();
        
        if(dr != null) {
        	this.oldCfg = dr;     
        	this.op = EDIT;
        	setTitle("Editing \"" + dr.getAlias() + "\"");
        	fillProperties(dr);
        }	
        else {        	        	
        	this.op = INSERT;
        	setTitle("New Driver");
        }
             
        this.setResizable(false);
        this.setLocationRelativeTo(null);
        this.setDefaultCloseOperation(DISPOSE_ON_CLOSE);
        this.setVisible(true);
    }
    
    private void initComponents() {
        aliasField = new javax.swing.JTextField();
        aliasLabel = new javax.swing.JLabel("Alias");
        addressLabel = new javax.swing.JLabel("Address");
        addressField = new javax.swing.JTextField();
        workloadPanel = new javax.swing.JPanel();
        phasesLabel = new javax.swing.JLabel("Phases");
        jScrollPane1 = new javax.swing.JScrollPane();
        phasesTable = new javax.swing.JTable();
        outOfOrderCheckBox = new javax.swing.JCheckBox("Send Events Out Of Order");
        okBtn = new javax.swing.JButton("OK", new ImageIcon("imgs/OK.png"));
        okBtn.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				try {
					if(phasesTable.getCellEditor() != null)
						phasesTable.getCellEditor().stopCellEditing();
					
					if(validateFields()) {
						WorkloadPhase workload[] = new WorkloadPhase[phases.size()]; 
						workload = phases.toArray(workload);					
						DriverConfig newCfg;
						try {
							int validatorPort = 0;
							double validatorSamplingRate = 0;
							if(validateCheckBox.isSelected()) {
								try {
									validatorPort = Integer.parseInt(validatorPortField.getText());
									validatorSamplingRate = Double.parseDouble((String)samplingComboBox.getSelectedItem());
								}
								catch (NumberFormatException nfe1) {
									JOptionPane.showMessageDialog(null, "Invalid value for validation port");
								}
								
							}
								
							newCfg = new DriverConfig(aliasField.getText(), InetAddress.getByName(addressField.getText()), workload, outOfOrderCheckBox.isSelected(), 
									InetAddress.getByName(serverAddressField.getText()), Integer.parseInt(serverPortField.getText()),
									validateCheckBox.isSelected(), InetAddress.getByName(validatorAddressField.getText()), 
									validatorPort , validatorSamplingRate
									);
							if (Controller.getInstance().checkDriverUniqueConstraint(oldCfg, newCfg)) {
								switch (op) {
								case EDIT:
									Controller.getInstance().updateDriver(oldCfg, newCfg);
									dispose();
									break;
								case INSERT:
									Controller.getInstance().addDriver(newCfg);
									dispose();
								}	
							}
							else
								JOptionPane.showMessageDialog(null, "New configuration violates unique constraint.", "Invalid Input", JOptionPane.ERROR_MESSAGE);
											
						} catch (UnknownHostException e2) {
							JOptionPane.showMessageDialog(null, "Invalid IP address.", "Invalid Input", JOptionPane.ERROR_MESSAGE);
						}
					}
					else {
						JOptionPane.showMessageDialog(null, "One or more required fields were not correctly filled.", "Invalid Input", JOptionPane.ERROR_MESSAGE);
					}
				} catch (NumberFormatException nfe) {					
					JOptionPane.showMessageDialog(null, "Invalid value at workload table");
				}
				
			}        	
        });
        cancelBtn = new javax.swing.JButton("Cancel", new ImageIcon("imgs/cancel.png"));
        cancelBtn.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {		
				dispose();
			}
        	
        });

        workloadPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Workload"));

        phasesTable.setModel(new DefaultTableModel(            
            new String [] {
                "Phase", "Type"
            	}, 1)  {
	        	@Override
	        	public boolean isCellEditable(int row, int column) {
	        			return false;
	        	}
        	});
        jScrollPane1.setViewportView(phasesTable);
        phasesTable.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				if (e.getClickCount() == 2) {
					JTable source = (JTable)e.getSource();
					if(source.isEnabled()) {
						int selected = source.getSelectedRow();
						if(selected > -1 && selected < phases.size())
							openPhaseDetail(phases.get(selected));							
					}					
				}}	
		});
        phasesTable.addMouseListener(new PopupListener(phasesPop));
        JMenuItem addPhaseMenuItem = new JMenuItem("Add...");
        JMenuItem deletePhaseMenuItem = new JMenuItem("Delete");
        JMenuItem copyPhaseMenuItem = new JMenuItem("Copy...");
        
        addPhaseMenuItem.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				openPhaseDetail(null);				
			}			
		});
        
        deletePhaseMenuItem.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				int index = phasesTable.getSelectedRow();

				if(index > -1 && index < phases.size()) {			
					removePhase(index);
				}
				else {
					JOptionPane.showMessageDialog(null, "Select a phase to delete");
				}				
			}			
		});
        
        copyPhaseMenuItem.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				
				int selected = phasesTable.getSelectedRow();				
				
				if(selected > -1 && selected < phases.size()) {
					WorkloadPhase copy = phases.get(selected);					
					PhaseDetail detail = openPhaseDetail(null);
					detail.fillProperties(copy);
				}						
				else
					JOptionPane.showMessageDialog(null, "Select a phase to copy");				

			}			
		});
        
        phasesPop.add(addPhaseMenuItem);
        phasesPop.add(deletePhaseMenuItem);
        phasesPop.add(copyPhaseMenuItem);
        
        javax.swing.GroupLayout workloadPanelLayout = new javax.swing.GroupLayout(workloadPanel);
        workloadPanel.setLayout(workloadPanelLayout);
        workloadPanelLayout.setHorizontalGroup(
            workloadPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(workloadPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(workloadPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 328, Short.MAX_VALUE)
                    .addComponent(phasesLabel))
                .addContainerGap())
        );
        workloadPanelLayout.setVerticalGroup(
            workloadPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(workloadPanelLayout.createSequentialGroup()
                .addComponent(phasesLabel)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 140, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(61, 61, 61))
        );


        serverAddressLabel = new JLabel("Server Address");
        serverAddressField = new JTextField();

        serverPortLabel = new JLabel("Server Port");
        serverPortField = new JTextField();

        validationPanel = new JPanel();
        validationPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Validation"));

        validateCheckBox = new JCheckBox("Send Events to an Online Validator");
        validateCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
            	if(validateCheckBox.isSelected()) {
            		validatorAddressField.setEnabled(true);
                    validatorPortField.setEnabled(true);
                    samplingComboBox.setEnabled(true);
            	}
            	else {
            		validatorAddressField.setEnabled(false);
                    validatorPortField.setEnabled(false);
                    samplingComboBox.setEnabled(false);
            	}
            }
        });

        validatorAddressLabel = new JLabel("Address");
        validatorAddressField = new JTextField();
        validatorAddressField.setEnabled(false);

        validatorPortLabel = new JLabel("Port");
        validatorPortField = new JTextField();
        validatorPortField.setEnabled(false);

        samplingComboBox = new JComboBox();
        samplingComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "1.0", "0.5", "0.25", "0.2", "0.1", "0.05", "0.025", "0.01", "0.001" }));
        samplingComboBox.setPreferredSize(new Dimension(25,20));
        samplingComboBox.setSelectedItem("0.1");
        samplingComboBox.setEnabled(false);

        samplingLabel = new JLabel("Sampling Rate");

        javax.swing.GroupLayout validationPanelLayout = new javax.swing.GroupLayout(validationPanel);
        validationPanel.setLayout(validationPanelLayout);
        validationPanelLayout.setHorizontalGroup(
            validationPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(validationPanelLayout.createSequentialGroup()
                .addGroup(validationPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(validationPanelLayout.createSequentialGroup()
                        .addContainerGap()
                        .addGroup(validationPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(validatorAddressField, javax.swing.GroupLayout.PREFERRED_SIZE, 170, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(validatorAddressLabel))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(validationPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(validatorPortLabel)
                            .addGroup(validationPanelLayout.createSequentialGroup()
                                .addComponent(validatorPortField, javax.swing.GroupLayout.PREFERRED_SIZE, 77, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addGap(6, 6, 6)
                                .addGroup(validationPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(samplingLabel)
                                    .addComponent(samplingComboBox, 0, 69, Short.MAX_VALUE)))))
                    .addComponent(validateCheckBox))
                .addContainerGap())
        );
        validationPanelLayout.setVerticalGroup(
            validationPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(validationPanelLayout.createSequentialGroup()
                .addGroup(validationPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addGroup(validationPanelLayout.createSequentialGroup()
                        .addComponent(validatorPortLabel)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(validatorPortField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(validationPanelLayout.createSequentialGroup()
                        .addComponent(validateCheckBox)
                        .addGap(7, 7, 7)
                        .addGroup(validationPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                            .addGroup(validationPanelLayout.createSequentialGroup()
                                .addComponent(validatorAddressLabel)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(validatorAddressField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                            .addGroup(validationPanelLayout.createSequentialGroup()
                                .addComponent(samplingLabel)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(samplingComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(validationPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(workloadPanel, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addGroup(layout.createSequentialGroup()
                        .addGap(10, 10, 10)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(serverAddressField, javax.swing.GroupLayout.PREFERRED_SIZE, 170, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(serverAddressLabel))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(serverPortLabel)
                            .addComponent(serverPortField, javax.swing.GroupLayout.PREFERRED_SIZE, 77, javax.swing.GroupLayout.PREFERRED_SIZE)))
                    .addGroup(layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(aliasField, javax.swing.GroupLayout.PREFERRED_SIZE, 170, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(aliasLabel))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(addressLabel)
                            .addComponent(addressField, javax.swing.GroupLayout.PREFERRED_SIZE, 180, javax.swing.GroupLayout.PREFERRED_SIZE)))
                    .addComponent(outOfOrderCheckBox)
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                        .addComponent(cancelBtn, javax.swing.GroupLayout.PREFERRED_SIZE, 99, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(okBtn, javax.swing.GroupLayout.PREFERRED_SIZE, 92, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(aliasLabel)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(aliasField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(addressLabel)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(addressField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(outOfOrderCheckBox)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(workloadPanel, javax.swing.GroupLayout.PREFERRED_SIZE, 198, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(serverAddressLabel)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(serverAddressField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(serverPortLabel)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(serverPortField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(validationPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(okBtn, javax.swing.GroupLayout.PREFERRED_SIZE, 34, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(cancelBtn, javax.swing.GroupLayout.PREFERRED_SIZE, 36, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );
        
        pack();
    }


	protected PhaseDetail openPhaseDetail(WorkloadPhase workloadPhase) {
    	return new PhaseDetail(this, workloadPhase);
	}

	public void fillProperties(DriverConfig dr) {
    	this.aliasField.setText(dr.getAlias());
    	this.addressField.setText(dr.getAddress().getHostAddress());
    	this.outOfOrderCheckBox.setSelected(dr.isSendOutOfOrder());
    	    
    	WorkloadPhase phases [] = dr.getWorkload();    	
    	for (int i = 0; i < phases.length; i++) {
    		addPhase(phases[i]);
		}
    	
    	this.serverAddressField.setText(dr.getServerAddress().getHostAddress());
    	this.serverPortField.setText(""+dr.getServerPort());
    	
    	if(dr.isValidationEnabled()) {
    		validateCheckBox.setSelected(true);
    		validatorAddressField.setText(dr.getValidatorAddress().getHostAddress());
    		validatorAddressField.setEnabled(true);
    		validatorPortField.setText(""+dr.getValidatorPort());
    		validatorPortField.setEnabled(true);
    		samplingComboBox.setSelectedItem(""+dr.getValidationSamplingRate());
    		samplingComboBox.setEnabled(true);
    	}
    }
    
    
    private boolean validateFields() {
    	try {
    		Integer.parseInt(this.serverPortField.getText());
    		
    		if(validateCheckBox.isSelected()) {
    			Integer.parseInt(this.validatorPortField.getText());
    			
    			if(this.validatorAddressField == null || this.validatorAddressField.getText().isEmpty())
    				return false;
    		}
    		
    	}
    	catch (NumberFormatException nfe) {
    		return false;
    	}
    	
    	    	
    	return (this.aliasField.getText() != null &&
    			!this.aliasField.getText().isEmpty() &&
    			this.addressField.getText() != null &&
    			!this.addressField.getText().isEmpty() &&
    			this.serverAddressField.getText() != null &&
    			!this.serverAddressField.getText().isEmpty() &&
    			phasesTable.getRowCount() > 1
    			);
    }

	public void updatePhase(WorkloadPhase oldCfg, WorkloadPhase newCfg) {
		int index = this.phases.indexOf(oldCfg);

		if(index > -1) {			
			this.phases.remove(index);				
			((DefaultTableModel)phasesTable.getModel()).removeRow(index);
			addPhase(index, newCfg);				
		}
		
		
		
	}

	public void addPhase(int index, WorkloadPhase phase) {
		this.phases.add(index,phase);
		
		DefaultTableModel model = (DefaultTableModel) this.phasesTable.getModel();
		if(phase instanceof SyntheticWorkloadPhase)
			model.insertRow(index, new Object[] {"Phase "+ (index+1), "Synthetic"});
		else if(phase instanceof ExternalFileWorkloadPhase)
			model.insertRow(index, new Object[] {"Phase "+ (index+1), "External File"});
	}
	
	public void addPhase(WorkloadPhase phase) {
		this.phases.add(phase);
		
		DefaultTableModel model = (DefaultTableModel) this.phasesTable.getModel();
		if(phase instanceof SyntheticWorkloadPhase)
			model.insertRow(model.getRowCount()-1, new Object[] {"Phase "+ (phases.indexOf(phase)+1), "Synthetic"});
		else if(phase instanceof ExternalFileWorkloadPhase)
			model.insertRow(model.getRowCount()-1, new Object[] {"Phase "+ (phases.indexOf(phase)+1), "External File"});		
	}

    private void removePhase(int index) {
    	if(index < phases.size()) {
    		phases.remove(index);						
    		((DefaultTableModel)phasesTable.getModel()).removeRow(index);
    		
    		for(int i=index; i<phasesTable.getRowCount()-1; i++)
    			phasesTable.setValueAt("Phase "+ (i+1), index, 0);	
    	}    	
	}
	
	public void disableGUI() {
		this.aliasField.setEnabled(false);
		this.addressField.setEnabled(false);
		this.outOfOrderCheckBox.setEnabled(false);
		this.phasesTable.setEnabled(false);
		this.serverAddressField.setEnabled(false);
		this.serverPortField.setEnabled(false);
		this.validateCheckBox.setEnabled(false);
		this.validatorAddressField.setEnabled(false);
		this.validatorPortField.setEnabled(false);
		this.samplingComboBox.setEnabled(false);
		this.okBtn.setEnabled(false);
	}
   
    
}

