
package controller.gui;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.Map.Entry;

import javax.swing.ImageIcon;
import javax.swing.JFileChooser;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.filechooser.FileNameExtensionFilter;
import javax.swing.table.DefaultTableModel;

import basic.EventType;
import basic.Globals;
import driver.ExternalFileWorkloadPhase;
import driver.SyntheticWorkloadPhase;
import driver.WorkloadPhase;

/**
 *
 * @author  Marcelo R.N. Mendes
 */
@SuppressWarnings("serial")
public class PhaseDetail extends ComponentDetail {
    
	private JFileChooser fileChooser;
	private ArrayList<EventType> syntheticTypes= new ArrayList<EventType>();
	DriverDetail parent;
	private WorkloadPhase oldCfg;
	
    /** Creates new form PhaseDetail */
    public PhaseDetail(DriverDetail parent, WorkloadPhase phase) {
        initComponents();
        
        if(phase != null) {
        	this.oldCfg = phase;     
        	this.op = EDIT;        
        	fillProperties(phase);
        }	
        else {        	        	
        	this.op = INSERT;
        	setTitle("New Phase");
        }
        
        fileChooser = new JFileChooser(Globals.APP_PATH);
        fileChooser.addChoosableFileFilter(new FileNameExtensionFilter("CSV data file", "csv"));
    	fileChooser.setAcceptAllFileFilterUsed(false);
    	fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
    	
    	this.parent = parent;
    }    
    private void initComponents() {
    	
//================================= GENERATED CODE ==========================================
    	 buttonGroup1 = new javax.swing.ButtonGroup();
         schemaTablePopup = new javax.swing.JPopupMenu();
         timestampRadioGroup = new javax.swing.ButtonGroup();
         eventTypeRadioGroup = new javax.swing.ButtonGroup();
         workloadPanel = new javax.swing.JPanel();
         externalFilePanel = new javax.swing.JPanel();
         externalFilePathField = new javax.swing.JTextField();
         fileBrowseBtn = new javax.swing.JButton();
         pathLabel = new javax.swing.JLabel();
         timestampsPanel = new javax.swing.JPanel();
         timestampRadioYES = new javax.swing.JRadioButton();
         timestampUnitCombo = new javax.swing.JComboBox();
         timestampRadioNO = new javax.swing.JRadioButton();
         timestampLabel1 = new javax.swing.JLabel();
         rateField = new javax.swing.JTextField();
         timestampLabel2 = new javax.swing.JLabel();
         timestampRadioYESNO = new javax.swing.JRadioButton();
         jLabel4 = new javax.swing.JLabel();
         jLabel5 = new javax.swing.JLabel();
         eventTypesPanel = new javax.swing.JPanel();
         eventTypeField = new javax.swing.JTextField();
         eventTypeLabel = new javax.swing.JLabel();
         eventTypeRadioNO = new javax.swing.JRadioButton();
         eventTypeRadioYES = new javax.swing.JRadioButton();
         externalFileRadio = new javax.swing.JRadioButton();
         syntheticPanel = new javax.swing.JPanel();
         schemaScroll = new javax.swing.JScrollPane();
         schemaTable = new javax.swing.JTable();
         schemaLabel = new javax.swing.JLabel();
         durationLabel = new javax.swing.JLabel();
         durationTextField = new javax.swing.JTextField();
         initialRateTextField = new javax.swing.JTextField();
         initialRateLabel = new javax.swing.JLabel();
         finalRateLabel = new javax.swing.JLabel();
         finalRateTextField = new javax.swing.JTextField();
         jLabel1 = new javax.swing.JLabel();
         jLabel2 = new javax.swing.JLabel();
         jLabel3 = new javax.swing.JLabel();
         syntheticRadio = new javax.swing.JRadioButton();
         okBtn = new javax.swing.JButton();
         cancelBtn = new javax.swing.JButton();

         setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
         setTitle("Phase Detail");
         setResizable(false);

         workloadPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Workload"));

         externalFilePanel.setBorder(javax.swing.BorderFactory.createTitledBorder(""));

         fileBrowseBtn.setText("...");

         pathLabel.setText("Path");

         timestampsPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Timestamps"));

         timestampRadioGroup.add(timestampRadioYES);         
         timestampRadioYES.setText("Data file contains timestamps, specified in");

         timestampUnitCombo.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Milliseconds", "Seconds", "Date/Time" }));

         timestampRadioGroup.add(timestampRadioNO);
         timestampRadioNO.setText("Data file DOES NOT contain timestamps.");

         timestampLabel1.setText("Events will be submitted at:");

         timestampLabel2.setText("events/sec");

         timestampRadioGroup.add(timestampRadioYESNO);
         timestampRadioYESNO.setText("Data file contains timestamps, but");

         jLabel4.setText("DO NOT use them for event submisssion!");

         jLabel5.setText("Use them for event submission!");

         javax.swing.GroupLayout timestampsPanelLayout = new javax.swing.GroupLayout(timestampsPanel);
         timestampsPanel.setLayout(timestampsPanelLayout);
         timestampsPanelLayout.setHorizontalGroup(
             timestampsPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
             .addGroup(timestampsPanelLayout.createSequentialGroup()
                 .addContainerGap()
                 .addGroup(timestampsPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                     .addGroup(timestampsPanelLayout.createSequentialGroup()
                         .addGap(21, 21, 21)
                         .addComponent(jLabel5))
                     .addGroup(timestampsPanelLayout.createSequentialGroup()
                         .addComponent(timestampRadioYES)
                         .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                         .addComponent(timestampUnitCombo, 0, 138, Short.MAX_VALUE))
                     .addGroup(timestampsPanelLayout.createSequentialGroup()
                         .addGroup(timestampsPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                             .addGroup(timestampsPanelLayout.createSequentialGroup()
                                 .addGap(21, 21, 21)
                                 .addComponent(jLabel4))
                             .addComponent(timestampRadioYESNO)
                             .addComponent(timestampRadioNO))
                         .addGroup(timestampsPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                             .addGroup(timestampsPanelLayout.createSequentialGroup()
                                 .addGap(17, 17, 17)
                                 .addComponent(rateField, javax.swing.GroupLayout.PREFERRED_SIZE, 65, javax.swing.GroupLayout.PREFERRED_SIZE)
                                 .addGap(7, 7, 7)
                                 .addComponent(timestampLabel2))
                             .addGroup(timestampsPanelLayout.createSequentialGroup()
                                 .addGap(18, 18, 18)
                                 .addComponent(timestampLabel1)))))
                 .addContainerGap())
         );
         timestampsPanelLayout.setVerticalGroup(
             timestampsPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
             .addGroup(timestampsPanelLayout.createSequentialGroup()
                 .addGroup(timestampsPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                     .addComponent(timestampRadioYES)
                     .addComponent(timestampUnitCombo, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                 .addGroup(timestampsPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                     .addGroup(javax.swing.GroupLayout.Alignment.LEADING, timestampsPanelLayout.createSequentialGroup()
                         .addGap(2, 2, 2)
                         .addComponent(jLabel5)
                         .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 17, Short.MAX_VALUE)
                         .addComponent(timestampRadioNO)
                         .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                         .addComponent(timestampRadioYESNO)
                         .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                         .addComponent(jLabel4)
                         .addContainerGap())
                     .addGroup(timestampsPanelLayout.createSequentialGroup()
                         .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                         .addComponent(timestampLabel1)
                         .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                         .addGroup(timestampsPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                             .addComponent(timestampLabel2)
                             .addComponent(rateField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                         .addGap(23, 23, 23))))
         );

         eventTypesPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Event Types"));

         eventTypeLabel.setText("All events are of the same type:");

         eventTypeRadioGroup.add(eventTypeRadioNO);
         eventTypeRadioNO.setText("Data File DOES NOT contain event types.");

         eventTypeRadioGroup.add(eventTypeRadioYES);         
         eventTypeRadioYES.setText("Data File contains event types");

         javax.swing.GroupLayout eventTypesPanelLayout = new javax.swing.GroupLayout(eventTypesPanel);
         eventTypesPanel.setLayout(eventTypesPanelLayout);
         eventTypesPanelLayout.setHorizontalGroup(
             eventTypesPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
             .addGroup(eventTypesPanelLayout.createSequentialGroup()
                 .addContainerGap()
                 .addGroup(eventTypesPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                     .addComponent(eventTypeRadioYES)
                     .addComponent(eventTypeRadioNO)
                     .addGroup(eventTypesPanelLayout.createSequentialGroup()
                         .addGap(21, 21, 21)
                         .addComponent(eventTypeLabel)
                         .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                         .addComponent(eventTypeField, javax.swing.GroupLayout.DEFAULT_SIZE, 189, Short.MAX_VALUE)))
                 .addContainerGap())
         );
         eventTypesPanelLayout.setVerticalGroup(
             eventTypesPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
             .addGroup(eventTypesPanelLayout.createSequentialGroup()
                 .addComponent(eventTypeRadioYES)
                 .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                 .addComponent(eventTypeRadioNO)
                 .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                 .addGroup(eventTypesPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                     .addComponent(eventTypeLabel)
                     .addComponent(eventTypeField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                 .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
         );

         javax.swing.GroupLayout externalFilePanelLayout = new javax.swing.GroupLayout(externalFilePanel);
         externalFilePanel.setLayout(externalFilePanelLayout);
         externalFilePanelLayout.setHorizontalGroup(
             externalFilePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
             .addGroup(externalFilePanelLayout.createSequentialGroup()
                 .addContainerGap()
                 .addGroup(externalFilePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                     .addComponent(timestampsPanel, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                     .addComponent(eventTypesPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                     .addComponent(pathLabel)
                     .addGroup(externalFilePanelLayout.createSequentialGroup()
                         .addComponent(externalFilePathField, javax.swing.GroupLayout.PREFERRED_SIZE, 340, javax.swing.GroupLayout.PREFERRED_SIZE)
                         .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                         .addComponent(fileBrowseBtn, javax.swing.GroupLayout.PREFERRED_SIZE, 29, javax.swing.GroupLayout.PREFERRED_SIZE)))
                 .addContainerGap())
         );
         externalFilePanelLayout.setVerticalGroup(
             externalFilePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
             .addGroup(externalFilePanelLayout.createSequentialGroup()
                 .addComponent(pathLabel)
                 .addGap(3, 3, 3)
                 .addGroup(externalFilePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                     .addComponent(fileBrowseBtn)
                     .addComponent(externalFilePathField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                 .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                 .addComponent(timestampsPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                 .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                 .addComponent(eventTypesPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                 .addContainerGap())
         );

         buttonGroup1.add(externalFileRadio);
         externalFileRadio.setText("External File");

         syntheticPanel.setBorder(javax.swing.BorderFactory.createTitledBorder(""));

         schemaTable.setComponentPopupMenu(schemaTablePopup);
         schemaTable.setModel(new javax.swing.table.DefaultTableModel(             
             new String [] {
                 "Type", "Columns", "Mix"
             }, 1
         ) {
             Class[] types = new Class [] {
                 java.lang.String.class, java.lang.String.class, java.lang.String.class
             };
             boolean[] canEdit = new boolean [] {
                 false, false, true
             };

             public Class getColumnClass(int columnIndex) {
                 return types [columnIndex];
             }

             public boolean isCellEditable(int rowIndex, int columnIndex) {
                 return canEdit [columnIndex];
             }
         });
         schemaTable.setMaximumSize(new java.awt.Dimension(150, 16));
         schemaScroll.setViewportView(schemaTable);

         schemaLabel.setText("Schema");

         durationLabel.setText("Duration");

         durationTextField.setToolTipText("");
         durationTextField.setPreferredSize(new java.awt.Dimension(75, 20));

         initialRateTextField.setToolTipText("");
         initialRateTextField.setPreferredSize(new java.awt.Dimension(75, 20));

         initialRateLabel.setText("Initial Rate");

         finalRateLabel.setText("Final Rate");

         finalRateTextField.setToolTipText("");
         finalRateTextField.setPreferredSize(new java.awt.Dimension(75, 20));

         jLabel1.setText("events/sec");

         jLabel2.setText("events/sec");

         jLabel3.setText("seconds");

         javax.swing.GroupLayout syntheticPanelLayout = new javax.swing.GroupLayout(syntheticPanel);
         syntheticPanel.setLayout(syntheticPanelLayout);
         syntheticPanelLayout.setHorizontalGroup(
             syntheticPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
             .addGroup(syntheticPanelLayout.createSequentialGroup()
                 .addContainerGap()
                 .addGroup(syntheticPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                     .addComponent(schemaScroll, javax.swing.GroupLayout.DEFAULT_SIZE, 397, Short.MAX_VALUE)
                     .addGroup(syntheticPanelLayout.createSequentialGroup()
                         .addGroup(syntheticPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                             .addComponent(durationLabel)
                             .addComponent(durationTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 65, javax.swing.GroupLayout.PREFERRED_SIZE))
                         .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                         .addComponent(jLabel3)
                         .addGap(21, 21, 21)
                         .addGroup(syntheticPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                             .addComponent(initialRateLabel)
                             .addComponent(initialRateTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 65, javax.swing.GroupLayout.PREFERRED_SIZE))
                         .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                         .addComponent(jLabel1)
                         .addGap(18, 18, 18)
                         .addGroup(syntheticPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                             .addGroup(syntheticPanelLayout.createSequentialGroup()
                                 .addComponent(finalRateTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 65, javax.swing.GroupLayout.PREFERRED_SIZE)
                                 .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                 .addComponent(jLabel2))
                             .addComponent(finalRateLabel)))
                     .addComponent(schemaLabel))
                 .addContainerGap())
         );
         syntheticPanelLayout.setVerticalGroup(
             syntheticPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
             .addGroup(syntheticPanelLayout.createSequentialGroup()
                 .addGroup(syntheticPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                     .addGroup(syntheticPanelLayout.createSequentialGroup()
                         .addGroup(syntheticPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                             .addComponent(durationLabel)
                             .addComponent(finalRateLabel))
                         .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                         .addGroup(syntheticPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                             .addComponent(finalRateTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                             .addComponent(jLabel2)
                             .addComponent(jLabel3)
                             .addComponent(durationTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                         .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                         .addComponent(schemaLabel))
                     .addGroup(syntheticPanelLayout.createSequentialGroup()
                         .addComponent(initialRateLabel)
                         .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                         .addGroup(syntheticPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                             .addComponent(initialRateTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                             .addComponent(jLabel1))))
                 .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                 .addComponent(schemaScroll, javax.swing.GroupLayout.PREFERRED_SIZE, 99, javax.swing.GroupLayout.PREFERRED_SIZE)
                 .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
         );

         buttonGroup1.add(syntheticRadio);
                  
         syntheticRadio.setText("Synthetic");

         javax.swing.GroupLayout workloadPanelLayout = new javax.swing.GroupLayout(workloadPanel);
         workloadPanel.setLayout(workloadPanelLayout);
         workloadPanelLayout.setHorizontalGroup(
             workloadPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
             .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, workloadPanelLayout.createSequentialGroup()
                 .addContainerGap()
                 .addGroup(workloadPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                     .addComponent(externalFilePanel, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                     .addComponent(syntheticPanel, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                     .addComponent(syntheticRadio, javax.swing.GroupLayout.Alignment.LEADING)
                     .addComponent(externalFileRadio, javax.swing.GroupLayout.Alignment.LEADING))
                 .addContainerGap())
         );
         workloadPanelLayout.setVerticalGroup(
             workloadPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
             .addGroup(workloadPanelLayout.createSequentialGroup()
                 .addComponent(syntheticRadio)
                 .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                 .addComponent(syntheticPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                 .addGap(18, 18, 18)
                 .addComponent(externalFileRadio)
                 .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                 .addComponent(externalFilePanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                 .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
         );

         okBtn.setText("OK");
         okBtn.setPreferredSize(null);

         cancelBtn.setText("Cancel");
         cancelBtn.setPreferredSize(null);

         javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
         getContentPane().setLayout(layout);
         layout.setHorizontalGroup(
             layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
             .addGroup(layout.createSequentialGroup()
                 .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                     .addComponent(workloadPanel, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                     .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                         .addContainerGap(255, Short.MAX_VALUE)
                         .addComponent(cancelBtn, javax.swing.GroupLayout.PREFERRED_SIZE, 100, javax.swing.GroupLayout.PREFERRED_SIZE)
                         .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                         .addComponent(okBtn, javax.swing.GroupLayout.PREFERRED_SIZE, 100, javax.swing.GroupLayout.PREFERRED_SIZE)))
                 .addContainerGap())
         );
         layout.setVerticalGroup(
             layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
             .addGroup(layout.createSequentialGroup()
                 .addComponent(workloadPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                 .addGap(18, 18, 18)
                 .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                     .addComponent(okBtn, javax.swing.GroupLayout.DEFAULT_SIZE, 34, Short.MAX_VALUE)
                     .addComponent(cancelBtn, javax.swing.GroupLayout.DEFAULT_SIZE, 34, Short.MAX_VALUE))
                 .addContainerGap())
         );

         pack();
//==============================  END OF GENERATED CODE =======================================
        
        

//==============================  EVENT HANDLING CODE =========================================        
        ItemListener radioListener1 = new ItemListener() {
			@Override
			public void itemStateChanged(ItemEvent e) {
				setPanelEnabled(syntheticPanel, syntheticRadio.isSelected());
				setPanelEnabled(externalFilePanel, externalFileRadio.isSelected());
				schemaTable.setEnabled(syntheticRadio.isSelected());								
				
				timestampUnitCombo.setEnabled(externalFileRadio.isSelected()&&timestampRadioYES.isSelected());            	            	
            	timestampLabel1.setEnabled(externalFileRadio.isSelected()&&!timestampRadioYES.isSelected());
            	timestampLabel2.setEnabled(externalFileRadio.isSelected()&&!timestampRadioYES.isSelected());
            	rateField.setEnabled(externalFileRadio.isSelected()&&!timestampRadioYES.isSelected());
            	
            	eventTypeField.setEnabled(eventTypeRadioNO.isSelected());
			}
        };        
        syntheticRadio.addItemListener(radioListener1);        
        externalFileRadio.addItemListener(radioListener1);
        
        ItemListener radioListener2 = new ItemListener() {
			@Override
			public void itemStateChanged(ItemEvent e) {           	
        		timestampUnitCombo.setEnabled(externalFileRadio.isSelected()&&timestampRadioYES.isSelected());            	            	
            	timestampLabel1.setEnabled(externalFileRadio.isSelected()&&!timestampRadioYES.isSelected());
            	timestampLabel2.setEnabled(externalFileRadio.isSelected()&&!timestampRadioYES.isSelected());
            	rateField.setEnabled(externalFileRadio.isSelected()&&!timestampRadioYES.isSelected());
            }
        };
        timestampRadioYES.addItemListener(radioListener2);
        timestampRadioNO.addItemListener(radioListener2);
        timestampRadioYESNO.addItemListener(radioListener2);
        
        ItemListener radioListener3 = new ItemListener() {
			@Override
			public void itemStateChanged(ItemEvent e) {             	
        		eventTypeField.setEnabled(eventTypeRadioNO.isSelected());
            }
        };
        eventTypeRadioYES.addItemListener(radioListener3);
        eventTypeRadioNO.addItemListener(radioListener3);      
        
        syntheticRadio.setSelected(true);
        timestampRadioYES.setSelected(true);
        eventTypeRadioYES.setSelected(true);
        
        fileBrowseBtn.addActionListener(new ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent e) {									
				fileChooser.showOpenDialog(null);
				
				if(fileChooser.getSelectedFile() != null) {
					externalFilePathField.setText(fileChooser.getSelectedFile().getPath());					
				}
			} 
		});	

        schemaTable.addMouseListener(new PopupListener(schemaTablePopup));
        JMenuItem addPhaseMenuItem = new JMenuItem("Add...");
        JMenuItem deletePhaseMenuItem = new JMenuItem("Delete");
        
        addPhaseMenuItem.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				openTypeDetail(null);				
			}			
		});
        
        deletePhaseMenuItem.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				int index = schemaTable.getSelectedRow();

				if(index > -1 && index < syntheticTypes.size()) {			
					syntheticTypes.remove(index);						
					((DefaultTableModel)schemaTable.getModel()).removeRow(index);
				}
				else {
					JOptionPane.showMessageDialog(null, "Select a type to delete");
				}				
			}			
		});
        
        schemaTablePopup.add(addPhaseMenuItem);
        schemaTablePopup.add(deletePhaseMenuItem);
        
        schemaTable.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				if (e.getClickCount() == 2) {
					JTable source = (JTable)e.getSource();
					if(source.isEnabled()) {
						int selected = source.getSelectedRow();
						if(selected > -1 && selected < syntheticTypes.size())
							openTypeDetail(syntheticTypes.get(selected));							
					}					
				}}	
		});
        
        okBtn.setIcon(new ImageIcon("imgs/OK.png"));
        okBtn.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				try {
					if(schemaTable.getCellEditor() != null)
						schemaTable.getCellEditor().stopCellEditing();
					
					if(validateFields()) {
						WorkloadPhase newCfg;
							
						if(syntheticRadio.isSelected()) { // synthetic data
							LinkedHashMap<EventType, Double> schema = new LinkedHashMap<EventType, Double>();
							
							DefaultTableModel model = (DefaultTableModel) schemaTable.getModel();
							
							for (int i = 0; i < model.getRowCount()-1; i++) {
								schema.put(syntheticTypes.get(i), Double.parseDouble((String) schemaTable.getValueAt(i, 2)));
							}
							
							newCfg = new SyntheticWorkloadPhase
							(
								Integer.parseInt(durationTextField.getText()),
								Double.parseDouble(initialRateTextField.getText()),
								Double.parseDouble(finalRateTextField.getText()),
								schema
							) ;
						}
						else  { // external file
							double rate = 1;
							
							if(!timestampRadioYES.isSelected())
								rate = Double.parseDouble(rateField.getText());
								
							newCfg = new ExternalFileWorkloadPhase
							(
								externalFilePathField.getText(), 
								!timestampRadioNO.isSelected(), 
								timestampRadioYES.isSelected(), 
								timestampUnitCombo.getSelectedIndex(), 
								rate, 
								eventTypeRadioYES.isSelected(), 
								eventTypeField.getText()
							);							
						}
									
						switch (op) {
						case EDIT:							
							parent.updatePhase(oldCfg, newCfg);
							dispose();
							break;
						case INSERT:							
							parent.addPhase(newCfg);
							dispose();
						}	
													
			
					}
					else {
						JOptionPane.showMessageDialog(null, "One or more required fields were not correctly filled.", "Invalid Input", JOptionPane.ERROR_MESSAGE);
					}
				} catch (NumberFormatException nfe) {
					JOptionPane.showMessageDialog(null, "Invalid value at Synthetic workload table");
				} catch (Exception exc) {
					JOptionPane.showMessageDialog(null, exc.getMessage());
				}
				
			}
      
        });
      
        cancelBtn.setIcon(new ImageIcon("imgs/Cancel.png"));
        cancelBtn.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {		
				dispose();
			}
        	
        });
                      
        this.setLocationRelativeTo(null); //screen center
        this.setVisible(true);
    }
    
    protected TypeDetail openTypeDetail(EventType type) {
    	return new TypeDetail(this, type);
	}
    
    public void fillProperties(WorkloadPhase phase) {
    	if(phase instanceof SyntheticWorkloadPhase) {
    		SyntheticWorkloadPhase synthPhase = (SyntheticWorkloadPhase)phase;
    		syntheticRadio.setSelected(true);
        	        	
    	   	this.durationTextField.setText(synthPhase.getDuration()+"");
        	this.initialRateTextField.setText(synthPhase.getInitialRate()+"");
        	this.finalRateTextField.setText(synthPhase.getFinalRate()+"");   

        	DefaultTableModel model = (DefaultTableModel) schemaTable.getModel();
        	int rowCount = model.getRowCount();		
        	for (int i = 0; i < rowCount; i++) {
        		model.removeRow(0);
        	}
        	if(synthPhase.getSchema() != null) {
        		for (Entry<EventType, Double> e : synthPhase.getSchema().entrySet()) {
        			this.syntheticTypes.add(e.getKey());
        			((DefaultTableModel)this.schemaTable.getModel()).addRow(
        					new Object[] {e.getKey().getName(), e.getKey().getAttributesNamesList(), ""+e.getValue()});        			
        		}
        		
        		((DefaultTableModel) this.schemaTable.getModel()).addRow(new Object[] {null, null});
        	}
    	}
    	else if(phase instanceof ExternalFileWorkloadPhase) {
    		ExternalFileWorkloadPhase filePhase = (ExternalFileWorkloadPhase) phase;
    		externalFileRadio.setSelected(true);
			
			this.externalFilePathField.setText(filePhase.getFilePath());
			if(filePhase.containsTimestamps()) {
				if(filePhase.isUsingTimestamps())
					timestampRadioYES.setSelected(true);
				else 
					timestampRadioYESNO.setSelected(true);
			}
			else
				timestampRadioNO.setSelected(true);				 
			timestampUnitCombo.setSelectedIndex(filePhase.getTimestampUnit());
			rateField.setText(filePhase.getEventSubmissionRate()+"");
			if(filePhase.containsEventTypes())				
				eventTypeRadioYES.setSelected(true);
			else
				eventTypeRadioNO.setSelected(true);
			eventTypeField.setText(filePhase.getSingleEventTypeName());
    	}
    	
    	this.validate();
    	this.repaint();
    }
    
    public void updateEventType(EventType oldType, EventType newType) {    	
		int index = this.syntheticTypes.indexOf(oldType);		
		
		if(index > -1) {
			String mix = (String) schemaTable.getValueAt(index, 2);			
			removeEventType(index);
			addEventType(index, newType);				
			schemaTable.setValueAt(mix, index, 2);
		}		
	}

	public void addEventType(int index, EventType newType) {
		this.syntheticTypes.add(index,newType);				
		((DefaultTableModel)this.schemaTable.getModel()).insertRow(index, new Object[] {newType.getName(), newType.getAttributesNamesList(), "1.0"});
	}
	
	public void addEventType(EventType newType) {		
		this.syntheticTypes.add(newType);
		DefaultTableModel model = ((DefaultTableModel)this.schemaTable.getModel());
		model.insertRow(model.getRowCount()-1, new Object[] {newType.getName(), newType.getAttributesNamesList(), "1.0"});
	}

    private void removeEventType(int index) {
    	syntheticTypes.remove(index);						
		((DefaultTableModel)schemaTable.getModel()).removeRow(index);
	}
    
	private boolean validateFields() {
		if(syntheticRadio.isSelected()) { // synthetic data
			return (this.durationTextField.getText() != null &&
	    			!this.durationTextField.getText().isEmpty() &&
	    			this.initialRateTextField.getText() != null &&
	    			!this.initialRateTextField.getText().isEmpty() &&
	    			this.finalRateTextField.getText() != null &&
	    			!this.finalRateTextField.getText().isEmpty() &&
	    			schemaTable.getRowCount() > 1
	    			);	
		}
		else { // external file
			return (externalFilePathField.getText()!= null &&
					!externalFilePathField.getText().isEmpty() &&
					(timestampRadioYES.isSelected() || 
					 (!timestampRadioYES.isSelected() && rateField.getText()!=null && !rateField.getText().isEmpty())
					) &&
					(eventTypeRadioYES.isSelected() ||
					(eventTypeRadioNO.isSelected() && eventTypeField.getText()!=null && !eventTypeField.getText().isEmpty())
					)
				   );
		}
		
	} 
	
	/**
	 * Sets the enabled state of all components and subcomponent of a given Panel 
	 */
	private void setPanelEnabled(JPanel c, boolean enabled) {
		Component comp;
		for (int i = 0; i < c.getComponentCount(); i++) {
			comp = c.getComponents()[i];
			
			if(comp instanceof JPanel)
				setPanelEnabled((JPanel)comp, enabled);
			else
				comp.setEnabled(enabled);
		}

	}
    
	 private javax.swing.ButtonGroup buttonGroup1;
	    private javax.swing.JButton cancelBtn;
	    private javax.swing.JLabel durationLabel;
	    private javax.swing.JTextField durationTextField;
	    private javax.swing.JTextField eventTypeField;
	    private javax.swing.JLabel eventTypeLabel;
	    private javax.swing.ButtonGroup eventTypeRadioGroup;
	    private javax.swing.JRadioButton eventTypeRadioNO;
	    private javax.swing.JRadioButton eventTypeRadioYES;
	    private javax.swing.JPanel eventTypesPanel;
	    private javax.swing.JPanel externalFilePanel;
	    private javax.swing.JTextField externalFilePathField;
	    private javax.swing.JRadioButton externalFileRadio;
	    private javax.swing.JButton fileBrowseBtn;
	    private javax.swing.JLabel finalRateLabel;
	    private javax.swing.JTextField finalRateTextField;
	    private javax.swing.JLabel initialRateLabel;
	    private javax.swing.JTextField initialRateTextField;
	    private javax.swing.JLabel jLabel1;
	    private javax.swing.JLabel jLabel2;
	    private javax.swing.JLabel jLabel3;
	    private javax.swing.JLabel jLabel4;
	    private javax.swing.JLabel jLabel5;
	    private javax.swing.JRadioButton timestampRadioYESNO;
	    private javax.swing.JButton okBtn;
	    private javax.swing.JLabel pathLabel;
	    private javax.swing.JTextField rateField;
	    private javax.swing.JLabel schemaLabel;
	    private javax.swing.JScrollPane schemaScroll;
	    private javax.swing.JTable schemaTable;
	    private javax.swing.JPopupMenu schemaTablePopup;
	    private javax.swing.JPanel syntheticPanel;
	    private javax.swing.JRadioButton syntheticRadio;
	    private javax.swing.JLabel timestampLabel1;
	    private javax.swing.JLabel timestampLabel2;
	    private javax.swing.ButtonGroup timestampRadioGroup;
	    private javax.swing.JRadioButton timestampRadioNO;
	    private javax.swing.JRadioButton timestampRadioYES;
	    private javax.swing.JComboBox timestampUnitCombo;
	    private javax.swing.JPanel timestampsPanel;
	    private javax.swing.JPanel workloadPanel;
    
}
