package data;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;

import basic.Event;
import basic.Globals;

public class CSVWriter {
	private BufferedWriter writer;

	
	public CSVWriter(String path) throws IOException {
		writer = new BufferedWriter(new FileWriter(path));
	}
	
	/**
	 * Writes an event along with its associated timestamp into a CSV file
	 * 
	 * @param e				An event, already in CSV representation
	 * @param timestamp		The timestamp of the event
	 * @throws IOException
	 */
	public void writeRecord(String e, long timestamp) throws IOException {
		writer.write(timestamp+Globals.CSV_SEPARATOR+e);
		writer.newLine();
	}
	
	/**
	 * Writes an event into a CSV file
	 * 
	 * @param e				An event, already in CSV representation		
	 * @throws IOException
	 */
	public void writeRecord(String e) throws IOException {
		writer.write(e);
		writer.newLine();
	}
	
	/**
	 * Writes an event along with its associated timestamp into a CSV file
	 * 
	 * @param e				An event, in an internal representation format
	 * @param timestamp		
	 * @throws IOException
	 */
	public void writeRecord(Event e, long timestamp) throws IOException {
		String line = "type:"+e.getType().getName();
		
		for (Object attributeValue : e.getAttributes().values()) {
			line+= Globals.CSV_SEPARATOR+attributeValue;
		}
		
		if(timestamp != 0)		
			writeRecord(line, timestamp);
		else
			writeRecord(line);
	}
		
	/**
	 * Writes an event into a CSV file
	 * 
	 * @param e				An event, in an internal representation format
	 * @throws IOException
	 */
	public void writeRecord(Event e) throws IOException {
		writeRecord(e, 0);
	}	
	
	public void closeFile() throws IOException{		
		writer.flush();
		writer.close();
	}
}
