package driver;

import java.io.Serializable;
import java.net.InetAddress;


@SuppressWarnings("serial")
public class DriverConfig implements Cloneable, Serializable{
	private String alias;
	private InetAddress address;
	private WorkloadPhase[] workload;
	private boolean sendOutOfOrder;
	
	private InetAddress serverAddress;
	private int serverPort;
	
	private boolean validationEnabled;
	private InetAddress validatorAddress;
	private int validatorPort;
	private double validationSamplingRate;
	
	/**
	 * 
	 * @param alias						An identifier for the Driver
	 * @param address					The address of the machine where the Driver should run
	 * @param workload					The workload specification of the Driver
	 * @param sendEventsOutOfOrder		Indicates if events must be sent out of order
	 * @param serverAddress				The address of the server to where events must be sent
	 * @param serverPort				The port in the server on which the events must be sent
	 * @param validate					Indicates if validation will be performed
	 * @param validatorAddress			The address of the Validator application
	 * @param validatorPort				The port on which the Validator listens for incoming events
	 * @param validationSamplingRate	The fraction of all events that will be sent for validation
	 */
	public DriverConfig(String alias, InetAddress address, WorkloadPhase[] workload, boolean sendEventsOutOfOrder, 
			InetAddress serverAddress, int serverPort,
			boolean validate, InetAddress validatorAddress, int validatorPort, double validationSamplingRate) {
		this.setAlias(alias);
		this.setAddress(address);
		this.setWorkload(workload);
		this.setSendOutOfOrder(sendEventsOutOfOrder);
		this.setServerAddress(serverAddress);
		this.setServerPort(serverPort);
		this.setValidationEnabled(validate);
		this.setValidatorAddress(validatorAddress);
		this.setValidatorPort(validatorPort);
		this.setValidationSamplingRate(validationSamplingRate);
	}
	
	public DriverConfig(String alias, InetAddress address, boolean sendEventsOutOfOrder, 
			InetAddress serverAddress, int serverPort, 
			boolean validate, InetAddress validatorAddress, int validatorPort, double validationSamplingRate) {
		this(alias, address, null, sendEventsOutOfOrder, serverAddress, serverPort, validate, validatorAddress, validatorPort, validationSamplingRate);
	}
	
	public DriverConfig(String alias, InetAddress address, 
			InetAddress serverAddress, int serverPort,
			boolean validate, InetAddress validatorAddress, int validatorPort, double validationSamplingRate) {
		this(alias, address, null, false, serverAddress, serverPort, validate, validatorAddress, validatorPort, validationSamplingRate);
	}

	public String getAlias() {
		return alias;
	}
	
	public void setAlias(String name) {
		this.alias = name;
	}
	
	public InetAddress getAddress() {
		return address;
	}
	
	public void setAddress(InetAddress address) {
		this.address = address;
	}

	public void setWorkload(WorkloadPhase[] workload) {
		this.workload = workload;
	}


	public WorkloadPhase[] getWorkload() {
		return workload;
	}

	public void setSendOutOfOrder(boolean sendOutOfOrder) {
		this.sendOutOfOrder = sendOutOfOrder;
	}

	public boolean isSendOutOfOrder() {
		return sendOutOfOrder;
	}
	
	public InetAddress getServerAddress() {
		return serverAddress;
	}
	
	public void setServerAddress(InetAddress serverAddress) {
		this.serverAddress = serverAddress;
	}
	
	public int getServerPort() {
		return serverPort;
	}
	
	public void setServerPort(int serverPort) {
		this.serverPort = serverPort;
	}
	
	public InetAddress getValidatorAddress() {
		return validatorAddress;
	}
	
	public void setValidatorAddress(InetAddress validatorAddress) {
		this.validatorAddress = validatorAddress;
	}
	
	public int getValidatorPort() {
		return validatorPort;
	}
	
	public void setValidatorPort(int validatorPort) {
		this.validatorPort = validatorPort;
	}
	
	public boolean isValidationEnabled() {
		return validationEnabled;
	}
	
	public void setValidationEnabled(boolean validationEnabled) {
		this.validationEnabled = validationEnabled;
	}
	
	public double getValidationSamplingRate() {
		return validationSamplingRate;
	}
	
	public void setValidationSamplingRate(double validationSamplingRate) {
		this.validationSamplingRate = validationSamplingRate;
	}
	
	@Override
	protected Object clone() {
		return new DriverConfig(this.alias, this.address, this.workload, this.sendOutOfOrder, 
				this.serverAddress, this.serverPort, 
				this.validationEnabled, this.validatorAddress, this.validatorPort, this.validationSamplingRate);
	}
	
	@Override
	public boolean equals(Object obj) {	
		DriverConfig comp = (DriverConfig) obj;
		
		boolean ret = (this.alias.equals(comp.alias) &&
				this.address.equals(comp.address) &&
				//this.workload.equals(comp.workload) &&
				this.sendOutOfOrder == comp.sendOutOfOrder &&
				this.serverAddress.equals(comp.serverAddress) &&
				this.serverPort == comp.serverPort &&
				this.validationEnabled == comp.validationEnabled);
		
				if(this.validationEnabled && comp.validationEnabled) {
					ret = ret && this.validatorAddress.equals(comp.validatorAddress) &&
									this.validatorPort == comp.validatorPort;	
				}

				return ret;
	}
	

}
