package driver;

import java.rmi.Remote;
import java.rmi.RemoteException;


import basic.InvalidStateException;
import basic.Status;

public interface DriverRemoteFunctions extends Remote {
	
	/**
	 * 
	 * Initializes a driver
	 * 
	 * @param dr				The configuration parameters of a Driver
	 * @param dataFilesDir		An optional parameter indicating the path to an external dataset file
	 * @return
	 * @throws InvalidStateException
	 * @throws RemoteException
	 */
	public boolean load (DriverConfig dr, String dataFilesDir) throws InvalidStateException, RemoteException;
	
	
	/**
	 * Start/Resume load submission on Drivers
	 * 
	 * @throws InvalidStateException
	 * @throws RemoteException
	 */
	public void start() throws InvalidStateException, RemoteException;
	
	
	/**
	 * Pause load submission on Drivers
	 * 
	 * @return
	 * @throws RemoteException
	 */
	public void pause() throws InvalidStateException, RemoteException;
	
	
	/**
	 * Stop load submission on Drivers
	 * 
	 * @return
	 * @throws RemoteException
	 */
	public void stop() throws InvalidStateException, RemoteException;
	
	
	/**
	 * Interrupts current phase and starts the next one. 
	 * If there is no more phases, the test is finished.
	 * 
	 * @throws RemoteException
	 */
	public void switchToNextPhase() throws InvalidStateException, RemoteException;
	
	
	/**
	 * Alter event submission rate
	 * 
	 * @param factor					factor by which event rates specified in configuration file must be multiplied
	 * @throws InvalidStateException
	 * @throws RemoteException
	 */
	public void alterRate(double factor) throws InvalidStateException, RemoteException;
	
	/**
	 * Gets the status of a remote driver and its progress
	 * 
	 * @return
	 * @throws RemoteException
	 */	
	public Status getStatus() throws RemoteException;
}
