package driver;

import java.util.LinkedHashMap;

import basic.EventType;

@SuppressWarnings("serial")
public class SyntheticWorkloadPhase extends WorkloadPhase {

	private int duration;
	private double initialRate;
	private double finalRate;
	private LinkedHashMap<EventType, Double> schema;
	
	public SyntheticWorkloadPhase(int duration, double initialRate, double finalRate, LinkedHashMap<EventType, Double> schema) {
		setDuration(duration);
		setInitialRate(initialRate);
		setFinalRate(finalRate);
		this.schema = schema;
	}
	
	public int getDuration() {
		return duration;
	}
	
	public double getInitialRate() {
		return initialRate;
	}
	
	public double getFinalRate() {
		return finalRate;
	}
	
	public LinkedHashMap<EventType, Double> getSchema() {
		return schema;
	}
	
	private void setInitialRate(double rate) {
		if(rate > 0) 
			this.initialRate = rate;
		else {
			if (rate == 0)
				this.initialRate = 1E-4; // 0.1 events/second
			else
				System.err.println("Invalid event rate.");
		}
	}
	
	private void setFinalRate(double finalRate) {
		if(finalRate > 0) 
			this.finalRate = finalRate;
		else {
			if (finalRate == 0)
				this.finalRate = 1E-4; //0.1 events/second
			else
				System.err.println("Invalid event rate.");
		}
		
	}
		
	private void setDuration(int duration) {
		this.duration = duration;
	}

}
