package random;
import java.util.Random;

/**
 * Class used for data generation.
 * Generates numbers uniformly distributed over a given range.
 * 
 * @author Marcelo R.N. Mendes
 *
 */
@SuppressWarnings("serial")
public class RandomUniformVariate extends Variate {	
	private  Random rnd = new Random();
	private double lower;
	private double upper;
	
	/**
	 * Class Constructor.
	 * 
	 * @param lower		the lower bound for the uniform distribution range (inclusive)	
	 * @param upper		the upper bound for the uniform distribution range (exclusive)
	 */
	public RandomUniformVariate(double lower, double upper) {
		this.lower = lower;
		this.upper = upper;
	}
	
	@Override
	public double generate() {
		return  (lower + rnd.nextDouble()*(upper-lower));
	}
	
	public double getLower() {
		return lower;
	}
	
	public double getUpper() {
		return upper;
	}

}
