package sink;

import java.io.Serializable;
import java.net.InetAddress;

@SuppressWarnings("serial")
public class SinkConfig implements Cloneable, Serializable{
	
	private String alias;
	private InetAddress address;
	private int port;
	private String outputStreamList[];
	
	private InetAddress serverAddress;	
	
	private boolean validationEnabled;
	private InetAddress validatorAddress;
	private int validatorPort;
	private double validationSamplingRate;
	
	
	/**
	 * 
	 * @param alias						A name for the Sink
	 * @param address					The address of the machine where the Sink should run
	 * @param port						The port to which the Sink will listen in order to process incoming events
	 * @param outputStreamsList			A list of streams to which the Sink is associated
	 * @param serverAddress				
	 * @param validationEnabled			Indicates if validation will be performed
	 * @param validatorAddress			The address of the Validator application
	 * @param validatorPort				The port on which the Validator listens for incoming events
	 * @param validationSamplingRate	The fraction of all events that will be sent for validation
	 */
	public SinkConfig(String alias, InetAddress address, int port, String[] outputStreamsList,
			InetAddress serverAddress,	boolean validationEnabled, InetAddress validatorAddress, int validatorPort, double validationSamplingRate) {
		setAlias(alias);
		setAddress(address);
		setPort(port);
		setOutputStreamList(outputStreamsList);
		this.setServerAddress(serverAddress);		
		this.setValidationEnabled(validationEnabled);
		this.setValidatorAddress(validatorAddress);
		this.setValidatorPort(validatorPort);
		this.setValidationSamplingRate(validationSamplingRate);
	}
	
	public String getAlias() {
		return alias;
	}
	
	public void setAlias(String alias) {
		this.alias = alias;
	}
	
	public InetAddress getAddress() {
		return address;
	}
	
	public void setAddress(InetAddress address) {
		this.address = address;
	}
	
	public int getPort() {
		return port;
	}
	
	public void setPort(int port) {
		this.port = port;
	}
	
	public String[] getOutputStreamList() {
		return outputStreamList;
	}
	
	public void setOutputStreamList(String[] inputStreamList) {
		if(inputStreamList != null)
			this.outputStreamList = inputStreamList;
		else
			this.outputStreamList = new String[0];
	}
	
	public int getStreamCount() {
		return this.outputStreamList.length;
	}
	
	public InetAddress getValidatorAddress() {
		return validatorAddress;
	}
	
	public void setValidatorAddress(InetAddress validatorAddress) {
		this.validatorAddress = validatorAddress;
	}
	
	public int getValidatorPort() {
		return validatorPort;
	}
	
	public void setValidatorPort(int validatorPort) {
		this.validatorPort = validatorPort;
	}
	
	public boolean isValidationEnabled() {
		return validationEnabled;
	}
	
	public void setValidationEnabled(boolean validationEnabled) {
		this.validationEnabled = validationEnabled;
	}
	
	public double getValidationSamplingRate() {
		return validationSamplingRate;
	}
	
	public void setValidationSamplingRate(double validationSamplingRate) {
		this.validationSamplingRate = validationSamplingRate;
	}
	
	public InetAddress getServerAddress() {
		return serverAddress;
	}
	
	public void setServerAddress(InetAddress serverAddress) {
		this.serverAddress = serverAddress;
	}

	
	@Override
	protected Object clone() throws CloneNotSupportedException {
		return new SinkConfig(this.alias, this.address, this.port, this.outputStreamList, this.serverAddress, this.validationEnabled, this.validatorAddress, this.validatorPort, this.validationSamplingRate);
	}
}
