package sink;

import java.io.IOException;
import java.rmi.Remote;
import java.rmi.RemoteException;


import basic.InvalidStateException;
import basic.Status;

public interface SinkRemoteFunctions extends Remote {
	
	/**
	 * Loads a Monitor:
	 * 1) Opens server socket and initializes worker threads to receive events from CEP server
	 * 2) Initializes logging, if enabled
	 * 3) Connects to online validator, if enabled
	 * 
	 * 
	 * @param mon			The configuration to be loaded
	 * @throws IOException
	 */
	public boolean load(SinkConfig mon) throws RemoteException, InvalidStateException;
	
	
	/**
	 * 
	 * Unloads a Monitor:
	 * 1) Closes worker threads and server socket 
	 * 2) Closes logger, if enabled
	 * 3) Disconnects from online validator, if online validation is enabled
	 */
	public void unload() throws RemoteException;
	
	
	
	/**
	 * Gets the status of a remote monitor and its progress
	 * 
	 * @return
	 * @throws RemoteException
	 */	
	public Status getStatus() throws RemoteException;
}
