package validation;

import java.awt.Dimension;
import java.text.DateFormat;
import java.text.DecimalFormat;
import java.util.Date;
import java.util.HashMap;

import javax.swing.JPanel;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.DateAxis;
import org.jfree.chart.axis.DateTickUnit;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.XYPlot;
import org.jfree.data.time.Millisecond;
import org.jfree.data.time.Second;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;

@SuppressWarnings("serial")
public class PerfChartPanel extends JPanel {		
	private HashMap<String, TimeSeries> seriesList;	
	private TimeSeriesCollection dataset;	
	NumberAxis yAxis;
	
	private static final int DEFAULT_ITEM_AGE = 60; // defines the number of periods a item is kept in the chart
	
	public PerfChartPanel(String chartTitle, String seriesList[], String yAxisName, DecimalFormat yAxisFormat) {				
			
		dataset = new TimeSeriesCollection();
		
		this.seriesList = new HashMap<String, TimeSeries>();
		
		 
		if(seriesList != null) {
			for (String seriesName : seriesList) {
				addSeries(seriesName);
			}	
		}						
		
		JFreeChart chart = ChartFactory.createTimeSeriesChart(null, "Time", yAxisName, dataset, true, true, true);						
		chart.setTitle(chartTitle);
		
		XYPlot plot = chart.getXYPlot();
        yAxis = (NumberAxis) plot.getRangeAxis();
       // yAxis.setAutoRange(false);
/*   
        yAxis.setLowerBound(0);
     //   yAxis.setUpperBound(15);
*/        yAxis.setNumberFormatOverride(yAxisFormat);
        
        DateAxis xAxis = (DateAxis) plot.getDomainAxis();
        xAxis.setTickUnit(new DateTickUnit(DateTickUnit.SECOND, 10, 
        		DateTickUnit.SECOND, 10, 
        		DateFormat.getTimeInstance()
        		));       
        
		ChartPanel chartPanel = new ChartPanel(chart);		
		chartPanel.setPreferredSize(new Dimension(400, 240));
		
		add(chartPanel);
	}

	public TimeSeries addSeries(String seriesName) {
		TimeSeries timeSeries = new TimeSeries(seriesName, Millisecond.class);
		timeSeries.setMaximumItemAge(DEFAULT_ITEM_AGE);
		this.seriesList.put(seriesName, timeSeries);
		dataset.addSeries(timeSeries);
		return timeSeries;
	}
	
	public void setYAxisRange(double min, double max) {
		yAxis.setLowerBound(min);
        yAxis.setUpperBound(max);
	}
	
	public void updateChart(String seriesName, long timestamp, double perfValue) {
		TimeSeries timeSeries = this.seriesList.get(seriesName); 
		if(timeSeries != null)
			timeSeries.addOrUpdate(new Second(new Date(timestamp)), perfValue);		
	}
	
	public static void updateChart(TimeSeries series, long timestamp, double perfValue) {
		series.addOrUpdate(new Second(new Date(timestamp)), perfValue);
	}
		
}	
	
